@@ -1,7 +1,7 @@
-# -*- coding: utf-8 -*-
+# -*- coding: utf-8; frozen_string_literal: true -*-
 #
 #--
-# Copyright (C) 2009-2016 Thomas Leitner <t_leitner@gmx.at>
+# Copyright (C) 2009-2019 Thomas Leitner <t_leitner@gmx.at>
 #
 # This file is part of kramdown which is licensed under the MIT.
 #++
@@ -47,16 +47,15 @@ module Kramdown
 
   # Return the data directory for kramdown.
   def self.data_dir
-    unless defined?(@@data_dir)
+    unless defined?(@data_dir)
       require 'rbconfig'
-      @@data_dir = File.expand_path(File.join(File.dirname(__FILE__), '..', '..', 'data', 'kramdown'))
-      @@data_dir = File.expand_path(File.join(RbConfig::CONFIG["datadir"], "kramdown")) if !File.exist?(@@data_dir)
-      raise "kramdown data directory not found! This is a bug, please report it!" unless File.directory?(@@data_dir)
+      @data_dir = File.expand_path(File.join(File.dirname(__FILE__), '..', '..', 'data', 'kramdown'))
+      @data_dir = File.expand_path(File.join(RbConfig::CONFIG["datadir"], "kramdown")) unless File.exist?(@data_dir)
+      raise "kramdown data directory not found! This is a bug, please report it!" unless File.directory?(@data_dir)
     end
-    @@data_dir
+    @data_dir
   end
 
-
   # The main interface to kramdown.
   #
   # This class provides a one-stop-shop for using kramdown to convert text into various output
@@ -84,7 +83,6 @@ module Kramdown
     # ::new) and the conversion phase.
     attr_reader :warnings
 
-
     # Create a new Kramdown document from the string +source+ and use the provided +options+. The
     # options that can be used are defined in the Options module.
     #
@@ -103,7 +101,8 @@ module Kramdown
       if Parser.const_defined?(parser)
         @root, @warnings = Parser.const_get(parser).parse(source, @options)
       else
-        raise Kramdown::Error.new("kramdown has no parser to handle the specified input format: #{@options[:input]}")
+        raise Kramdown::Error, "kramdown has no parser to handle the specified " \
+          "input format: #{@options[:input]}"
       end
     end
 
@@ -122,7 +121,7 @@ module Kramdown
       end
     end
 
-    def inspect #:nodoc:
+    def inspect # :nodoc:
       "<KD:Document: options=#{@options.inspect} root=#{@root.inspect} warnings=#{@warnings.inspect}>"
     end
 
@@ -138,4 +137,3 @@ module Kramdown
   end
 
 end
-
