@@ -1,7 +1,7 @@
-# -*- coding: utf-8 -*-
+# -*- coding: utf-8; frozen_string_literal: true -*-
 #
 #--
-# Copyright (C) 2009-2015 Thomas Leitner <t_leitner@gmx.at>
+# Copyright (C) 2009-2019 Thomas Leitner <t_leitner@gmx.at>
 #
 # This file is part of kramdown which is licensed under the MIT.
 #++
@@ -14,6 +14,14 @@ module Kramdown
   # kramdown only uses this one class for representing all available elements in an element tree
   # (paragraphs, headers, emphasis, ...). The type of element can be set via the #type accessor.
   #
+  # The root of a kramdown element tree has to be an element of type :root. It needs to have certain
+  # option keys set so that conversions work correctly. If only a part of a tree should be
+  # converted, duplicate the root node and assign the #children appropriately, e.g:
+  #
+  #   root = doc.root
+  #   new_root = root.dup
+  #   new_root.children = [root.children[0]]  # assign new array with elements to convert
+  #
   # Following is a description of all supported element types.
   #
   # Note that the option :location may contain the start line number of an element in the source
@@ -42,6 +50,7 @@ module Kramdown
   #
   # :options:: This key may be used to store options that were set during parsing of the document.
   #
+  # :footnote_count:: This key stores the number of actually referenced footnotes of the document.
   #
   # === :blank
   #
@@ -482,17 +491,16 @@ module Kramdown
     # The child elements of this element.
     attr_accessor :children
 
-
     # Create a new Element object of type +type+. The optional parameters +value+, +attr+ and
     # +options+ can also be set in this constructor for convenience.
     def initialize(type, value = nil, attr = nil, options = nil)
-      @type, @value, @attr, @options = type, value, (Utils::OrderedHash.new.merge!(attr) if attr), options
+      @type, @value, @attr, @options = type, value, attr, options
       @children = []
     end
 
-    # The attributes of the element. Uses an Utils::OrderedHash to retain the insertion order.
+    # The attributes of the element.
     def attr
-      @attr ||= Utils::OrderedHash.new
+      @attr ||= {}
     end
 
     # The options hash for the element. It is used for storing arbitray options.
@@ -500,14 +508,19 @@ module Kramdown
       @options ||= {}
     end
 
-    def inspect #:nodoc:
-      "<kd:#{@type}#{@value.nil? ? '' : ' ' + @value.inspect} #{@attr.inspect}#{options.empty? ? '' : ' ' + @options.inspect}#{@children.empty? ? '' : ' ' + @children.inspect}>"
+    def inspect # :nodoc:
+      "<kd:#{@type}" \
+        "#{value.nil? ? '' : ' value=' + value.inspect}" \
+        "#{attr.empty? ? '' : ' attr=' + attr.inspect}" \
+        "#{options.empty? ? '' : ' options=' + options.inspect}" \
+        "#{children.empty? ? '' : ' children=' + children.inspect}>"
     end
 
     CATEGORY = {} # :nodoc:
-    [:blank, :p, :header, :blockquote, :codeblock, :ul, :ol, :li, :dl, :dt, :dd, :table, :td, :hr].each {|b| CATEGORY[b] = :block}
+    [:blank, :p, :header, :blockquote, :codeblock, :ul, :ol, :li, :dl, :dt, :dd,
+     :table, :td, :hr].each {|b| CATEGORY[b] = :block }
     [:text, :a, :br, :img, :codespan, :footnote, :em, :strong, :entity, :typographic_sym,
-     :smart_quote, :abbreviation].each {|b| CATEGORY[b] = :span}
+     :smart_quote, :abbreviation].each {|b| CATEGORY[b] = :span }
 
     # Return the category of +el+ which can be :block, :span or +nil+.
     #
@@ -517,6 +530,22 @@ module Kramdown
       CATEGORY[el.type] || el.options[:category]
     end
 
+    # syntactic sugar to simplify calls such as +Kramdown::Element.category(el) == :block+ with
+    # +el.block?+.
+    #
+    # Returns boolean true or false.
+    def block?
+      (CATEGORY[type] || options[:category]) == :block
+    end
+
+    # syntactic sugar to simplify calls such as +Kramdown::Element.category(el) == :span+ with
+    # +el.span?+.
+    #
+    # Returns boolean true or false.
+    def span?
+      (CATEGORY[type] || options[:category]) == :span
+    end
+
   end
 
 end
