@@ -1,7 +1,7 @@
-# -*- coding: utf-8 -*-
+# -*- coding: utf-8; frozen_string_literal: true -*-
 #
 #--
-# Copyright (C) 2009-2015 Thomas Leitner <t_leitner@gmx.at>
+# Copyright (C) 2009-2019 Thomas Leitner <t_leitner@gmx.at>
 #
 # This file is part of kramdown which is licensed under the MIT.
 #++
@@ -52,8 +52,14 @@ module Kramdown
       def initialize(source, options)
         @source = source
         @options = Kramdown::Options.merge(options)
-        @root = Element.new(:root, nil, nil, :encoding => (source.encoding rescue nil), :location => 1,
-                            :options => {}, :abbrev_defs => {}, :abbrev_attr => {})
+        @root = Element.new(:root, nil, nil, encoding: (source.encoding rescue nil), location: 1,
+                            options: {}, abbrev_defs: {}, abbrev_attr: {})
+
+        @root.options[:abbrev_defs].default_proc = @root.options[:abbrev_attr].default_proc =
+          lambda do |h, k|
+            k_mod = k.gsub(/[\s\p{Z}]+/, " ")
+            k != k_mod ? h[k_mod] : nil
+          end
         @warnings = []
         @text_type = :text
       end
@@ -83,19 +89,19 @@ module Kramdown
       # Add the given warning +text+ to the warning array.
       def warning(text)
         @warnings << text
-        #TODO: add position information
+        # TODO: add position information
       end
 
       # Modify the string +source+ to be usable by the parser (unifies line ending characters to
       # +\n+ and makes sure +source+ ends with a new line character).
       def adapt_source(source)
-        if source.respond_to?(:encode)
-          if !source.valid_encoding?
-            raise "The source text contains invalid characters for the used encoding #{source.encoding}"
-          end
-          source = source.encode('UTF-8')
+        unless source.valid_encoding?
+          raise "The source text contains invalid characters for the used encoding #{source.encoding}"
         end
-        source.gsub(/\r\n?/, "\n").chomp + "\n"
+        source = source.encode('UTF-8')
+        source.gsub!(/\r\n?/, "\n")
+        source.chomp!
+        source << "\n"
       end
 
       # This helper method adds the given +text+ either to the last element in the +tree+ if it is a
@@ -105,7 +111,8 @@ module Kramdown
         if last && last.type == type
           last.value << text
         elsif !text.empty?
-          tree.children << Element.new(type, text, nil, :location => (last && last.options[:location] || tree.options[:location]))
+          location = (last && last.options[:location] || tree.options[:location])
+          tree.children << Element.new(type, text, nil, location: location)
         end
       end
 
@@ -113,16 +120,12 @@ module Kramdown
       # method works correctly under Ruby 1.8 and Ruby 1.9.
       def extract_string(range, strscan)
         result = nil
-        if strscan.string.respond_to?(:encoding)
-          begin
-            enc = strscan.string.encoding
-            strscan.string.force_encoding('ASCII-8BIT')
-            result = strscan.string[range].force_encoding(enc)
-          ensure
-            strscan.string.force_encoding(enc)
-          end
-        else
-          result = strscan.string[range]
+        begin
+          enc = strscan.string.encoding
+          strscan.string.force_encoding('ASCII-8BIT')
+          result = strscan.string[range].force_encoding(enc)
+        ensure
+          strscan.string.force_encoding(enc)
         end
         result
       end
