@@ -1,7 +1,7 @@
-# -*- coding: utf-8 -*-
+# -*- coding: utf-8; frozen_string_literal: true -*-
 #
 #--
-# Copyright (C) 2009-2016 Thomas Leitner <t_leitner@gmx.at>
+# Copyright (C) 2009-2019 Thomas Leitner <t_leitner@gmx.at>
 #
 # This file is part of kramdown which is licensed under the MIT.
 #++
@@ -18,11 +18,12 @@ module Kramdown
         start_line_number = @src.current_line_number
         @src.pos += @src.matched_size
         begin
-          @tree.children << Element.new(:entity, ::Kramdown::Utils::Entities.entity(@src[1] || (@src[2] && @src[2].to_i) || @src[3].hex),
-                                        nil, :original => @src.matched, :location => start_line_number)
+          value = ::Kramdown::Utils::Entities.entity(@src[1] || @src[2]&.to_i || @src[3].hex)
+          @tree.children << Element.new(:entity, value,
+                                        nil, original: @src.matched, location: start_line_number)
         rescue ::Kramdown::Error
           @tree.children << Element.new(:entity, ::Kramdown::Utils::Entities.entity('amp'),
-                                        nil, :location => start_line_number)
+                                        nil, location: start_line_number)
           add_text(@src.matched[1..-1])
         end
       end
