@@ -1,7 +1,7 @@
-# -*- coding: utf-8 -*-
+# -*- coding: utf-8; frozen_string_literal: true -*-
 #
 #--
-# Copyright (C) 2009-2016 Thomas Leitner <t_leitner@gmx.at>
+# Copyright (C) 2009-2019 Thomas Leitner <t_leitner@gmx.at>
 #
 # This file is part of kramdown which is licensed under the MIT.
 #++
@@ -30,12 +30,12 @@ module Kramdown
       # Used for parsing the first line of a list item or a definition, i.e. the line with list item
       # marker or the definition marker.
       def parse_first_list_line(indentation, content)
-        if content =~ self.class::LIST_ITEM_IAL_CHECK
+        if content.match?(self.class::LIST_ITEM_IAL_CHECK)
           indentation = 4
         else
-          while content =~ /^ *\t/
+          while content.match?(/^ *\t/)
             temp = content.scan(/^ */).first.length + indentation
-            content.sub!(/^( *)(\t+)/) {$1 << " "*(4 - (temp % 4) + ($2.length - 1)*4)}
+            content.sub!(/^( *)(\t+)/) { $1 << " " * (4 - (temp % 4) + ($2.length - 1) * 4) }
           end
           indentation += content[/^ */].length
         end
@@ -44,23 +44,24 @@ module Kramdown
         [content, indentation, *PARSE_FIRST_LIST_LINE_REGEXP_CACHE[indentation]]
       end
 
+      PATTERN_TAIL = /[\t| ].*?\n/
 
-      LIST_START_UL = /^(#{OPT_SPACE}[+*-])([\t| ].*?\n)/
-      LIST_START_OL = /^(#{OPT_SPACE}\d+\.)([\t| ].*?\n)/
+      LIST_START_UL = /^(#{OPT_SPACE}[+*-])(#{PATTERN_TAIL})/
+      LIST_START_OL = /^(#{OPT_SPACE}\d+\.)(#{PATTERN_TAIL})/
       LIST_START = /#{LIST_START_UL}|#{LIST_START_OL}/
 
       # Parse the ordered or unordered list at the current location.
       def parse_list
         start_line_number = @src.current_line_number
         type, list_start_re = (@src.check(LIST_START_UL) ? [:ul, LIST_START_UL] : [:ol, LIST_START_OL])
-        list = new_block_el(type, nil, nil, :location => start_line_number)
+        list = new_block_el(type, nil, nil, location: start_line_number)
 
         item = nil
         content_re, lazy_re, indent_re = nil
         eob_found = false
         nested_list_found = false
         last_is_blank = false
-        while !@src.eos?
+        until @src.eos?
           start_line_number = @src.current_line_number
           if last_is_blank && @src.check(HR_START)
             break
@@ -68,17 +69,18 @@ module Kramdown
             eob_found = true
             break
           elsif @src.scan(list_start_re)
-            item = Element.new(:li, nil, nil, :location => start_line_number)
-            item.value, indentation, content_re, lazy_re, indent_re = parse_first_list_line(@src[1].length, @src[2])
+            list.options[:first_list_marker] ||= @src[1].strip
+            item = Element.new(:li, nil, nil, location: start_line_number)
+            item.value, indentation, content_re, lazy_re, indent_re =
+              parse_first_list_line(@src[1].length, @src[2])
             list.children << item
 
-            item.value.sub!(self.class::LIST_ITEM_IAL) do |match|
+            item.value.sub!(self.class::LIST_ITEM_IAL) do
               parse_attribute_list($1, item.options[:ial] ||= {})
               ''
             end
 
-            list_start_re = (type == :ul ? /^( {0,#{[3, indentation - 1].min}}[+*-])([\t| ].*?\n)/ :
-                             /^( {0,#{[3, indentation - 1].min}}\d+\.)([\t| ].*?\n)/)
+            list_start_re = fetch_pattern(type, indentation)
             nested_list_found = (item.value =~ LIST_START)
             last_is_blank = false
             item.value = [item.value]
@@ -86,14 +88,14 @@ module Kramdown
             result.sub!(/^(\t+)/) { " " * 4 * $1.length }
             indentation_found = result.sub!(indent_re, '')
             if !nested_list_found && indentation_found && result =~ LIST_START
-              item.value << ''
+              item.value << +''
               nested_list_found = true
             elsif nested_list_found && !indentation_found && result =~ LIST_START
               result = " " * (indentation + 4) << result
             end
             item.value.last << result
             last_is_blank = false
-          elsif result = @src.scan(BLANK_LINE)
+          elsif (result = @src.scan(BLANK_LINE))
             nested_list_found = true
             last_is_blank = true
             item.value.last << result
@@ -106,7 +108,7 @@ module Kramdown
 
         last = nil
         list.children.each do |it|
-          temp = Element.new(:temp, nil, nil, :location => it.options[:location])
+          temp = Element.new(:temp, nil, nil, location: it.options[:location])
 
           env = save_env
           location = it.options[:location]
@@ -119,26 +121,24 @@ module Kramdown
 
           it.children = temp.children
           it.value = nil
-          next if it.children.size == 0
+
+          it_children = it.children
+          next if it_children.empty?
 
           # Handle the case where an EOB marker is inserted by a block IAL for the first paragraph
-          it.children.delete_at(1) if it.children.first.type == :p &&
-            it.children.length >= 2 && it.children[1].type == :eob && it.children.first.options[:ial]
+          it_children.delete_at(1) if it_children.first.type == :p &&
+            it_children.length >= 2 && it_children[1].type == :eob && it_children.first.options[:ial]
 
-          if it.children.first.type == :p &&
-              (it.children.length < 2 || it.children[1].type != :blank ||
-               (it == list.children.last && it.children.length == 2 && !eob_found)) &&
+          if it_children.first.type == :p &&
+              (it_children.length < 2 || it_children[1].type != :blank ||
+              (it == list.children.last && it_children.length == 2 && !eob_found)) &&
               (list.children.last != it || list.children.size == 1 ||
-               list.children[0..-2].any? {|cit| !cit.children.first || cit.children.first.type != :p || cit.children.first.options[:transparent]})
-            it.children.first.children.first.value << "\n" if it.children.size > 1 && it.children[1].type != :blank
-            it.children.first.options[:transparent] = true
+              list.children[0..-2].any? {|cit| !cit.children.first || cit.children.first.type != :p || cit.children.first.options[:transparent] })
+            it_children.first.children.first.value << "\n" if it_children.size > 1 && it_children[1].type != :blank
+            it_children.first.options[:transparent] = true
           end
 
-          if it.children.last.type == :blank
-            last = it.children.pop
-          else
-            last = nil
-          end
+          last = (it_children.last.type == :blank ? it_children.pop : nil)
         end
 
         @tree.children << last if !last.nil? && !eob_found
@@ -147,14 +147,14 @@ module Kramdown
       end
       define_parser(:list, LIST_START)
 
-
-      DEFINITION_LIST_START = /^(#{OPT_SPACE}:)([\t| ].*?\n)/
+      DEFINITION_LIST_START = /^(#{OPT_SPACE}:)(#{PATTERN_TAIL})/
 
       # Parse the ordered or unordered list at the current location.
       def parse_definition_list
         children = @tree.children
-        if !children.last || (children.length == 1 && children.last.type != :p ) ||
-            (children.length >= 2 && children[-1].type != :p && (children[-1].type != :blank || children[-1].value != "\n" || children[-2].type != :p))
+        if !children.last || (children.length == 1 && children.last.type != :p) ||
+            (children.length >= 2 && children[-1].type != :p &&
+             (children[-1].type != :blank || children[-1].value != "\n" || children[-2].type != :p))
           return false
         end
 
@@ -165,9 +165,10 @@ module Kramdown
           para = @tree.children.pop
           first_as_para = true
         end
-        deflist.options[:location] = para.options[:location] # take location from preceding para which is the first definition term
-        para.children.first.value.split(/\n/).each do |term|
-          el = Element.new(:dt, nil, nil, :location => @src.current_line_number)
+        # take location from preceding para which is the first definition term
+        deflist.options[:location] = para.options[:location]
+        para.children.first.value.split("\n").each do |term|
+          el = Element.new(:dt, nil, nil, location: @src.current_line_number)
           term.sub!(self.class::LIST_ITEM_IAL) do
             parse_attribute_list($1, el.options[:ial] ||= {})
             ''
@@ -182,31 +183,32 @@ module Kramdown
         content_re, lazy_re, indent_re = nil
         def_start_re = DEFINITION_LIST_START
         last_is_blank = false
-        while !@src.eos?
+        until @src.eos?
           start_line_number = @src.current_line_number
           if @src.scan(def_start_re)
-            item = Element.new(:dd, nil, nil, :location => start_line_number)
+            item = Element.new(:dd, nil, nil, location: start_line_number)
             item.options[:first_as_para] = first_as_para
-            item.value, indentation, content_re, lazy_re, indent_re = parse_first_list_line(@src[1].length, @src[2])
+            item.value, indentation, content_re, lazy_re, indent_re =
+              parse_first_list_line(@src[1].length, @src[2])
             deflist.children << item
 
-            item.value.sub!(self.class::LIST_ITEM_IAL) do |match|
+            item.value.sub!(self.class::LIST_ITEM_IAL) do |_match|
               parse_attribute_list($1, item.options[:ial] ||= {})
               ''
             end
 
-            def_start_re = /^( {0,#{[3, indentation - 1].min}}:)([\t| ].*?\n)/
+            def_start_re = fetch_pattern(:dl, indentation)
             first_as_para = false
             last_is_blank = false
           elsif @src.check(EOB_MARKER)
             break
           elsif (result = @src.scan(content_re)) || (!last_is_blank && (result = @src.scan(lazy_re)))
-            result.sub!(/^(\t+)/) { " "*($1 ? 4*$1.length : 0) }
+            result.sub!(/^(\t+)/) { " " * ($1 ? 4 * $1.length : 0) }
             result.sub!(indent_re, '')
             item.value << result
             first_as_para = false
             last_is_blank = false
-          elsif result = @src.scan(BLANK_LINE)
+          elsif (result = @src.scan(BLANK_LINE))
             first_as_para = true
             item.value << result
             last_is_blank = true
@@ -221,35 +223,64 @@ module Kramdown
 
           parse_blocks(it, it.value)
           it.value = nil
-          next if it.children.size == 0
+          it_children = it.children
+          next if it_children.empty?
 
-          if it.children.last.type == :blank
-            last = it.children.pop
-          else
-            last = nil
-          end
+          last = (it_children.last.type == :blank ? it_children.pop : nil)
 
-          if it.children.first && it.children.first.type == :p && !it.options.delete(:first_as_para)
-            it.children.first.children.first.value << "\n" if it.children.size > 1
-            it.children.first.options[:transparent] = true
+          if it_children.first && it_children.first.type == :p && !it.options.delete(:first_as_para)
+            it_children.first.children.first.value << "\n" if it_children.size > 1
+            it_children.first.options[:transparent] = true
           end
         end
 
-        if @tree.children.length >= 1 && @tree.children.last.type == :dl
-          @tree.children[-1].children.concat(deflist.children)
-        elsif @tree.children.length >= 2 && @tree.children[-1].type == :blank && @tree.children[-2].type == :dl
-          @tree.children.pop
-          @tree.children[-1].children.concat(deflist.children)
+        children = @tree.children
+        if children.length >= 1 && children.last.type == :dl
+          children[-1].children.concat(deflist.children)
+        elsif children.length >= 2 && children[-1].type == :blank &&
+            children[-2].type == :dl
+          children.pop
+          children[-1].children.concat(deflist.children)
         else
-          @tree.children << deflist
+          children << deflist
         end
 
-        @tree.children << last if !last.nil?
+        children << last if last
 
         true
       end
       define_parser(:definition_list, DEFINITION_LIST_START)
 
+      private
+
+      # precomputed patterns for indentations 1..4 and fallback expression
+      # to compute pattern when indentation is outside the 1..4 range.
+      def fetch_pattern(type, indentation)
+        case type
+        when :ul
+          case indentation
+          when 1 then /^( {0}[+*-])(#{PATTERN_TAIL})/o
+          when 2 then /^( {0,1}[+*-])(#{PATTERN_TAIL})/o
+          when 3 then /^( {0,2}[+*-])(#{PATTERN_TAIL})/o
+          else /^( {0,3}[+*-])(#{PATTERN_TAIL})/o
+          end
+        when :ol
+          case indentation
+          when 1 then /^( {0}\d+\.)(#{PATTERN_TAIL})/o
+          when 2 then /^( {0,1}\d+\.)(#{PATTERN_TAIL})/o
+          when 3 then /^( {0,2}\d+\.)(#{PATTERN_TAIL})/o
+          else /^( {0,3}\d+\.)(#{PATTERN_TAIL})/o
+          end
+        when :dl
+          case indentation
+          when 1 then /^( {0}:)(#{PATTERN_TAIL})/o
+          when 2 then /^( {0,1}:)(#{PATTERN_TAIL})/o
+          when 3 then /^( {0,2}:)(#{PATTERN_TAIL})/o
+          else /^( {0,3}:)(#{PATTERN_TAIL})/o
+          end
+        end
+      end
+
     end
   end
 end
