@@ -1,7 +1,7 @@
-# -*- coding: utf-8 -*-
+# -*- coding: utf-8; frozen_string_literal: true -*-
 #
 #--
-# Copyright (C) 2009-2016 Thomas Leitner <t_leitner@gmx.at>
+# Copyright (C) 2009-2019 Thomas Leitner <t_leitner@gmx.at>
 #
 # This file is part of kramdown which is licensed under the MIT.
 #++
@@ -27,13 +27,13 @@ module Kramdown
       def entity_to_str(e, original = nil)
         entity_output = @options[:entity_output]
 
-        if e.char.respond_to?(:encoding) && entity_output == :as_char &&
+        if entity_output == :as_char &&
             (c = e.char.encode(@root.options[:encoding]) rescue nil) &&
-            ((c = e.char) == '"' || !ESCAPE_MAP.has_key?(c))
+            ((c = e.char) == '"' || !ESCAPE_MAP.key?(c))
           c
         elsif (entity_output == :as_input || entity_output == :as_char) && original
           original
-        elsif (entity_output == :symbolic || ESCAPE_MAP.has_key?(e.char)) && !e.name.nil?
+        elsif (entity_output == :symbolic || ESCAPE_MAP.key?(e.char)) && !e.name.nil?
           "&#{e.name};"
         else # default to :numeric
           "&##{e.code_point};"
@@ -42,7 +42,11 @@ module Kramdown
 
       # Return the HTML representation of the attributes +attr+.
       def html_attributes(attr)
-        attr.map {|k,v| v.nil? || (k == 'id' && v.strip.empty?) ? '' : " #{k}=\"#{escape_html(v.to_s, :attribute)}\"" }.join('')
+        return '' if attr.empty?
+
+        attr.map do |k, v|
+          v.nil? || (k == 'id' && v.strip.empty?) ? '' : " #{k}=\"#{escape_html(v.to_s, :attribute)}\""
+        end.join
       end
 
       # :stopdoc:
@@ -50,16 +54,12 @@ module Kramdown
         '<' => '&lt;',
         '>' => '&gt;',
         '&' => '&amp;',
-        '"' => '&quot;'
+        '"' => '&quot;',
       }
       ESCAPE_ALL_RE = /<|>|&/
       ESCAPE_TEXT_RE = Regexp.union(REXML::Parsers::BaseParser::REFERENCE_RE, /<|>|&/)
       ESCAPE_ATTRIBUTE_RE = Regexp.union(REXML::Parsers::BaseParser::REFERENCE_RE, /<|>|&|"/)
-      ESCAPE_RE_FROM_TYPE = {
-        :all => ESCAPE_ALL_RE,
-        :text => ESCAPE_TEXT_RE,
-        :attribute => ESCAPE_ATTRIBUTE_RE
-      }
+      ESCAPE_RE_FROM_TYPE = {all: ESCAPE_ALL_RE, text: ESCAPE_TEXT_RE, attribute: ESCAPE_ATTRIBUTE_RE}
       # :startdoc:
 
       # Escape the special HTML characters in the string +str+. The parameter +type+ specifies what
@@ -67,7 +67,14 @@ module Kramdown
       # entities, :text - all special HTML characters except the quotation mark but no entities and
       # :attribute - all special HTML characters including the quotation mark but no entities.
       def escape_html(str, type = :all)
-        str.gsub(ESCAPE_RE_FROM_TYPE[type]) {|m| ESCAPE_MAP[m] || m}
+        str.gsub(ESCAPE_RE_FROM_TYPE[type]) {|m| ESCAPE_MAP[m] || m }
+      end
+
+      REDUNDANT_LINE_BREAK_REGEX = /([\p{Han}\p{Hiragana}\p{Katakana}]+)\n([\p{Han}\p{Hiragana}\p{Katakana}]+)/u
+      def fix_cjk_line_break(str)
+        while str.gsub!(REDUNDANT_LINE_BREAK_REGEX, '\1\2')
+        end
+        str
       end
 
     end
